<?php
/**
 *
 *===================================================================
 *
 *  Crunch -- Core File
 *-------------------------------------------------------------------
 *	Script info:
 * Version:		1.0.0 - "Moonshine" - RC 2
 * Copyright:	(c) 2009 - Obsidian
 * License:		http://opensource.org/licenses/gpl-license.php  |  GNU Public License v2
 * Package:		Includes
 *
 *===================================================================
 * 
 */

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
    exit;
}

abstract class crunch
{
	/**
	 * Is this a development version of Crunch?
	 * @var boolean
	 */
	public static $dev = false;

/**
 * Class constants
 */

	/**
	 * crunch::make() constant for Crunch creation success
	 * @var constant
	 */
	const POST_SUCCESS = 0;

	/**
	 * crunch::make() constant for Crunch edit success
	 * @var constant
	 */
	const EDIT_SUCCESS = 1;

	/**
	 * crunch::make() constant for Crunch creation failure
	 * @var constant
	 */
	const POST_FAILED = 2;

	/**
	 * crunch::make() constant for Crunch edit failure
	 * @var constant
	 */
	const EDIT_FAILED = 3;

	/**
	 * crunch::make() constant for Crunch edit failure due to nonexistent Crunch being specified
	 * @var constant
	 */
	const EDIT_FAILED_NO_SUCH_CRUNCH = 4;

	/**
	 * crunch::make() constant for Crunch edit failure due to no actual change being made
	 * @var constant
	 */
	const EDIT_FAILED_NO_CHANGE = 5;

	/**
	 * Failure handler
	 * @var constant
	 */
	const FAIL = 'Q09QWVJJR0hUIElMTEVHQUxMWSBSRU1PVkVEISAtLSBNT0QgSVMgRElTQUJMRUQgVU5USUwgQ09QWVJJR0hUIElTIFJFUExBQ0VE';

	/**
	 * Installation verification checksum
	 * @var constant
	 */
	const CHECKSUM = 'f171e4253d6d4049bf9a04b40576bcbb';
	
/**
 * Class methods
 */
	
	/**
	 * Class initiator function.  Call this first before doing anything.
	 * 		It will kill uninstalled access attempts and try to force installation/upgrades automatically 
	 * 		 if new updated files are detected, and will automatically load needed classes/files.
	 * @return void
	 */
	public static function init()
	{
		global $user, $template, $config, $phpEx, $phpbb_root_path, $table_prefix;

		// Check if we've run self::init() already
		if(defined('CRUNCH_INIT'))
		{
			return;
		}

		// Start initiating stuffs.
		define('CRUNCH_VERSION', '1.0.0RC2');
		define('CRUNCH_BIG_VERSION', 'Crunch 1.0.0 - &quot;Moonshine&quot; - RC 2');

		// Define DB tables
		define('CRUNCH_TABLE', $table_prefix . 'crunch');
		define('CRUNCH_TAGS_TABLE', $table_prefix . 'crunch_tags');
		
		$user->add_lang(array('mods/lang_crunch', 'viewtopic'));
		$template->assign_var('S_CRUNCH_LOADED', true);

		/**
		 * Check if the DB component is installed, and current.
		 * Should we run around in circles screaming, or just push the user to the install page? O_o
		 * Also, let's check to see if the install directory is still present.  If it is, disable, and if it's a founder, tell them to delete the thing already.
		 */
		if(!defined('IN_ADMIN'))
		{
			// If not installed or old version...
			if(!isset($config['crunch_version']) || version_compare($config['crunch_version'], CRUNCH_VERSION, '<'))
			{
				// If we're a founder and install file is present...
				if((int) $user->data['user_type'] === USER_FOUNDER)
				{
					$template->assign_var('CRUNCH_FAIL', ((!isset($config['crunch_version'])) ? $user->lang['CRUNCH_NOT_INSTALLED'] : $user->lang['CRUNCH_DB_NOT_CURRENT']));
				}
				// Disable Crunch if not disabled already.
				if(!defined('DISABLE_CRUNCH'))
				{
					define('DISABLE_CRUNCH', true);
					$template->assign_var('DISABLE_CRUNCH', true);
				}
			}

			// Okay, doth our install dir still exist?
			// We disable this check in DEBUG_EXTRA.
			if(@file_exists("{$phpbb_root_path}install_crunch") && !defined('DISABLE_CRUNCH') && !defined('DEBUG_EXTRA'))
			{
				if((int) $user->data['user_type'] === USER_FOUNDER)
				{
					// IDIOT...Get rid of the directory already, would you?  >_<
					$template->assign_var('CRUNCH_FAIL', $user->lang['CRUNCH_INSTALL_DIR_PRESENT']);
				}
				if(!defined('DISABLE_CRUNCH'))
				{
					define('DISABLE_CRUNCH', true);
					$template->assign_var('DISABLE_CRUNCH', true);
				}
			}

			// Check to see if Crunch has been disabled and we're not in the ACP.
			if($config['crunch_enabled'] != true && (!defined('DISABLE_CRUNCH')))
			{
				if((int) $user->data['user_type'] === USER_FOUNDER)
				{
					$template->assign_var('CRUNCH_FAIL', $user->lang['CRUNCH_DISABLED']);
				}
				define('DISABLE_CRUNCH', true);
				$template->assign_var('DISABLE_CRUNCH', true);
			}
		}

		// Okay now...what files do we need to import?
		include_once("{$phpbb_root_path}includes/functions_display.$phpEx");

		// Check if Crunch auto-disabled itself here.
		if(!defined('DISABLE_CRUNCH'))
		{
			self::setup();
			if(!defined('crunch::CHECKSUM') || (md5(CRUNCH_INSTALL) !== self::CHECKSUM))
			{
				self::fail();
			}
		}
	}

/**
 * Crunch loading methods
 */

	/**
	 * Does a complete load of the Crunch specified, assigns template vars, and all that jazz.
	 * @param string $crunch_name - Crunch tag for specifying a Crunch to load.
	 * @return void
	 */
	public static function load($crunch_name)
	{
		global $template;
		$crunch = self::get($crunch_name);
		if($crunch === false)
		{
			$template->assign_var('S_CRUNCH_FAIL', true);
		}
		else
		{
			$template->assign_block_vars('crunchrow', $crunch);
		}
	}

	/**
	 * Method that searches for Crunches that are assigned to specified tag, then randomly selects one and pulls its data
	 * @param string $crunch_name - Crunch tag for specifying a Crunch to load.
	 * @return mixed - Array of values to pass to template if a valid Crunch is found assigned to that tag, false if Crunch does not exist or no such tag is assigned.
	 */
	public static function get($crunch_name)
	{
		global $db;
	
		$sql = 'SELECT crunch_id
		FROM ' . CRUNCH_TAGS_TABLE . "
		WHERE tag_word = '" . $db->sql_escape($crunch_name) . "'
			AND tag_enable = '1'";
		
		$result = $db->sql_query($sql);
	
		while ($row = $db->sql_fetchrow($result))
		{
			if ($row == false)
			{
				return false;
			}
			$crunch_ids[] = $row['crunch_id'];
		}
		$db->sql_freeresult($result);

		return self::get_by_id($crunch_ids[rand(0, sizeof($crunch_ids) - 1)]);
	}

	/**
	 * A method that pulls a specified Crunch's data, using the Crunch ID as a reference for what to load.
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to pull data up for.
	 * @return mixed - Array of values to pass to template if a valid Crunch is found, false if Crunch does not exist.
	 */
	public static function get_by_id($crunch_id)
	{
		static $crunch_cache = array();
		global $db, $config, $user;
		$crunch_data = self::query($crunch_id);
	
		//! React to presence of data...
		if ($crunch_data == false)
		{
			return false;
		}
	
		//! Quickly dump the poster_id into another var for readability.
		$poster_id = $crunch_data['poster_id'];

		#! Emulating the viewtopic user cache system, to be nicer to PHP.  :P
		if(!isset($crunch_cache[$poster_id]))
		{
			$crunch_cache[$poster_id] = array(
				'avatar'			=> ($user->optionget('viewavatars')) ? get_user_avatar($crunch_data['user_avatar'], $crunch_data['user_avatar_type'], $crunch_data['user_avatar_width'], $crunch_data['user_avatar_height']) : '',
				'username'			=> $crunch_data['username'],
				'user_colour'		=> $crunch_data['user_colour'],
				'rank_title'		=> '',
				'rank_image'		=> '',
				'rank_image_src'	=> '',
			);
			//! Get poster rank if we've not gotten it for this user already, also.
			get_user_rank($crunch_data['user_rank'], $crunch_data['user_posts'], $crunch_cache[$poster_id]['rank_title'], $crunch_cache[$poster_id]['rank_image'], $crunch_cache[$poster_id]['rank_image_src']);
		}
		$return = array(
			//! Prep some template vars.
			'POST_AUTHOR_FULL'			=> get_username_string('full', $poster_id, $crunch_data['username'], $crunch_data['user_colour']),
			'POST_AUTHOR_COLOUR'		=> get_username_string('colour', $poster_id, $crunch_data['username'], $crunch_data['user_colour']),
			'POST_AUTHOR'				=> get_username_string('username', $poster_id, $crunch_data['username'], $crunch_data['user_colour']),
			'U_POST_AUTHOR'				=> get_username_string('profile', $poster_id, $crunch_data['username'], $crunch_data['user_colour']),
			'POSTER_AVATAR'				=> $crunch_cache[$poster_id]['avatar'],
		
			'POST_SUBJECT'				=> censor_text($crunch_data['crunch_subject']),
			'MESSAGE'					=> generate_text_for_display($crunch_data['crunch_text'], $crunch_data['crunch_uid'], $crunch_data['crunch_bitfield'], $crunch_data['crunch_options']),
		
			'RANK_TITLE'				=> $crunch_cache[$poster_id]['rank_title'],
			'RANK_IMG'					=> $crunch_cache[$poster_id]['rank_image'],
			'RANK_IMG_SRC'				=> $crunch_cache[$poster_id]['rank_image_src'],
			'POST_ID'					=> $crunch_data['crunch_id'],
		);
	
		return $return;
	}
	
	/**
	 * Loads an index of Crunches
	 * @return mixed - Array of subarrays containing Crunch data, or false if no Crunches exist.
	 */
	public static function load_list()
	{
		global $db;
		$sql = 'SELECT crunch_id, crunch_subject
			FROM ' . CRUNCH_TABLE . '
			ORDER BY crunch_id ASC';
		$result = $db->sql_query($sql);
		while($row = $db->sql_fetchrow($result))
		{
			// No results?
			if ($row == false)
			{
				return false;
			}
			$crunch_data[] = $row;
		}
		$db->sql_freeresult($result);
		return (isset($crunch_data)) ? $crunch_data : false;
	}

/**
 * Crunch manipulation methods
 */

	/**
	 * Gets a Crunch's data by ID, for editing purposes.
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to pull data up for.
	 * @return mixed - Array of values to pass to template if a valid Crunch is found, false if Crunch does not exist.
	 */
	public static function edit($crunch_id)
	{
		global $db, $config, $user;
		$crunch_data = self::query($crunch_id);
	
		//! React to presence of data...
		if ($crunch_data === false)
		{
			return false;
		}
		
		$crunch_post = generate_text_for_edit($crunch_data['crunch_text'], $crunch_data['crunch_uid'], $crunch_data['crunch_options']);
		
		$return = array(
			//! Prep some vars.
			'poster_id'						=> (int) $crunch_data['poster_id'],
		
			'crunch_subject'				=> $crunch_data['crunch_subject'],
			'crunch_text'					=> $crunch_post['text'],
			
			'crunch_parse_bbcode'			=> ($crunch_post['allow_bbcode']) ? true : false,
			'crunch_parse_urls'				=> ($crunch_post['allow_urls']) ? true : false,
			'crunch_parse_smilies'			=> ($crunch_post['allow_smilies']) ? true : false,
		
			'crunch_id'						=> (int) $crunch_data['crunch_id'],
		);
	
		return $return;
	}

	/**
	 * DB query method to save code redundancy between edit-loading and display-loading.
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to pull data up for.
	 * @return array - SQL data of Crunch to be pulled up.
	 */
	private static function query($crunch_id)
	{
		global $db;
		$sql_ary = array(
			'SELECT'	=> 'cr.*, u.user_id, u.username, u.user_colour,	u.user_avatar, 
						u.user_avatar_type, u.user_avatar_width, u.user_avatar_height, u.user_rank, u.user_posts',
	
			'FROM'		=> array(
				CRUNCH_TABLE	=> 'cr',
			),
	
			'LEFT_JOIN'	=> array(
				array(
					'FROM'		=> array(USERS_TABLE => 'u'),
					'ON'		=> 'u.user_id = cr.poster_id',
				),
			),
	
			'WHERE'				=> 'cr.crunch_id = ' . (int) $crunch_id,
		);
		$sql = $db->sql_build_query('SELECT', $sql_ary);
	
		$result = $db->sql_query($sql);
		$crunch_data = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		return $crunch_data;
	}

	/**
	 * Will create a new crunch or update it if it already exists
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to pull data up for. Defaults to false, set as false if not editing.
	 * @param integer $user_id - User's ID that will be the "author" of the Crunch.
	 * @param string $text - Crunch's text, in raw format, assuming it was run through utf8_normalize_nfc() right after request_var().
	 * @param string $subject - Crunch's subject, assuming it was run through utf8_normalize_nfc() right after request_var()
	 * @param integer $options - Reference variable.  Include it but leave it blank, mmkay?
	 * @param boolean $allow_bbcodes - Allow BBcodes to be used in the Crunch
	 * @param boolean $allow_urls - Allow Magic URL's to be used in the Crunch
	 * @param boolean $allow_smilies - Allow smilies to be used in the Crunch
	 * @return constant - Returns whether the Crunch creation was a success or not, using special constants for identifying what down.
	 */
	public static function make($crunch_id = false, $user_id, $text, $subject, &$options, $allow_bbcode = true, $allow_urls = true, $allow_smilies = true)
	{
		global $db;

		// First things first.  Need to determine if we're editing, and then prepare our data for insert.
		if($crunch_id === false)
		{
			// Create new Crunch..
			$uid = $bitfield = $options = '';
			generate_text_for_storage($text, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);
			
			$sql_ary = array(
				'poster_id'			=> (int) $user_id,
				'crunch_text'		=> (string) $text,
				'crunch_subject'	=> (string) $subject,

				'crunch_bitfield'	=> $bitfield,
				'crunch_uid'		=> $uid,
				'crunch_options'	=> $options,
			);
	
			$sql = 'INSERT INTO ' . CRUNCH_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
			$return = $db->sql_query($sql);
			
			return ($return === true) ? self::POST_SUCCESS : self::POST_FAILED;
			
		}
		else
		{
			// Edit old Crunch..
			$uid = $bitfield = $options = '';
			generate_text_for_storage($text, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);
			
			$sql_ary = array(
				'poster_id'			=> (int) $user_id,
				'crunch_text'		=> (string) $text,
				'crunch_subject'	=> (string) $subject,

				'crunch_bitfield'	=> $bitfield,
				'crunch_uid'		=> $uid,
				'crunch_options'	=> $options,
			);
			
			$sql = 'UPDATE ' . CRUNCH_TABLE . '
				SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
				WHERE crunch_id = ' . (int) $crunch_id;
			$return = $db->sql_query($sql);

			return ($return === true) ? self::EDIT_SUCCESS : self::EDIT_FAILED;
		}
	}

	/**
	 * Removes a Crunch totally from existence, along with associated Crunch tags.
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to remove.
	 * @return boolean - Whether or not the deletion was a success.
	 */
	public static function delete($crunch_id)
	{
		global $db;
		$sql = 'DELETE FROM ' . CRUNCH_TABLE . ' WHERE crunch_id = ' . (int) $crunch_id;
		$db->sql_query($sql);
		
		$sql = 'DELETE FROM ' . CRUNCH_TAGS_TABLE . ' WHERE crunch_id = ' . (int) $crunch_id;
		$db->sql_query($sql);
		return true;
	}

	/**
	 * Checks to see if a Crunch with a specified Crunch ID exists.
	 * @param integer $crunch_id - Crunch ID that identifies the Crunch that we want to remove.
	 * @return boolean - Whether or not the Crunch exists.
	 */
	public static function exists($crunch_id)
	{
		global $db;

		$sql = 'SELECT COUNT(crunch_id) AS crunches 
			FROM ' . CRUNCH_TABLE . '
				WHERE crunch_id = ' . (int) $crunch_id;
		$result = $db->sql_query($sql);
		$results = (int) $db->sql_fetchfield('crunches');
		$db->sql_freeresult($result);

		return (($results > 0) ? true : false);
	}

/**
 * Crunch tag manipulation methods
 */

	/**
	 * Loads an index list of all Crunch tags available.
	 * @return mixed - Array of subarrays, containing the results, or false if no Crunch Tags exist.
	 */
	public static function load_tag_list()
	{
		global $db;

		$sql_ary = array(
			'SELECT'	=> 'crt.*, cr.crunch_subject',
			
			'FROM'		=> array(
				CRUNCH_TAGS_TABLE		=> 'crt',
			),
			
			'LEFT_JOIN'	=> array(
				array(
					'FROM'		=> array(CRUNCH_TABLE => 'cr'),
					'ON'		=> 'cr.crunch_id = crt.crunch_id',
				),
			),
		);

		$sql = $db->sql_build_query('SELECT', $sql_ary);
		$result = $db->sql_query($sql);

		while($row = $db->sql_fetchrow($result))
		{
			if($row == false)
			{
				return false;
			}
			$crunch_tags[] = $row;
		}
		$db->sql_freeresult($result);
		
		return (isset($crunch_tags)) ? $crunch_tags : false;
	}

	/**
	 * Assigns a Crunch tag to a certain Crunch if not already in existence.
	 * @param integer $crunch_id - Crunch ID for specifying specific tag manipulation
	 * @param string $crunch_name - Name of Crunch tag to be manipulated.
	 * @param boolean $crunch_status - Whether the tag is enabled or not on creation.
	 * @return boolean - True on success, false on failure or tag already existing for specified Crunch.
	 */
	public static function make_tag($crunch_id, $crunch_name, $crunch_status = true)
	{
		global $db;
		if((self::tag_exists($crunch_id, $crunch_name) === true) || ($crunch_status !== true && $crunch_status !== false))
		{
			return false;
		}
		
		$sql_ary = array(
			'crunch_id'			=> (int) $crunch_id,
			'tag_word'			=> (string) $crunch_name,
			'tag_enable'		=> (bool) $crunch_status,
		);
		$sql = 'INSERT INTO ' . CRUNCH_TAGS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		return $db->sql_query($sql);
	}

	/**
	 * Deletes a specified Crunch tag.
	 * @param integer $crunch_id - Crunch ID for specifying specific tag manipulation.
	 * @param string $crunch_name - Name of Crunch tag to be manipulated.
	 * @return boolean - True on success, false on failure.
	 */
	public static function delete_tag($crunch_id, $crunch_name)
	{
		global $db;
		$sql = 'DELETE FROM ' . CRUNCH_TAGS_TABLE . ' WHERE crunch_id = ' . (int) $crunch_id . "
					AND tag_word = '" . $db->sql_escape($crunch_name) . "'";
		return $db->sql_query($sql);
	}

	/**
	 * Deletes a specified Crunch tag using the Tag's ID.
	 * @param integer $tag_id - Tag ID for specifying specific tag manipulation
	 * @return boolean - True on success, false on failure.
	 */
	public static function delete_tag_by_id($tag_id)
	{
		global $db;
		$sql = 'DELETE FROM ' . CRUNCH_TAGS_TABLE . ' WHERE tag_id = ' . (int) $tag_id;
		return $db->sql_query($sql);
	}

	/**
	 * Enables/disables Crunch tags for use.
	 * @param integer $crunch_id - Crunch ID for specifying specific tag manipulation
	 * @param string $crunch_name - Name of Crunch tag to be manipulated.
	 * @param boolean $crunch_status - Whether the tag should be enabled or not.
	 * @return boolean - True if successful, false if unsuccessful.
	 */
	public static function tag_enable($crunch_id, $crunch_name, $crunch_status = true)
	{
		global $db;
		if((self::tag_exists($crunch_id, $crunch_name) === false) || ($crunch_status !== true && $crunch_status !== false))
		{
			return false;
		}

		$sql = 'UPDATE ' . CRUNCH_TAGS_TABLE . "
			SET tag_enable = '" . (int) $crunch_status . "'
			WHERE crunch_id = " . (int) $crunch_id . "
				AND tag_word = '" . $db->sql_escape($crunch_name) . "'";
		return $db->sql_query($sql);
	}

	/**
	 * Checks to see if a Crunch tag exists given the tag word and Crunch ID.
	 * @param integer $crunch_id - Crunch ID for specifying specific tag manipulation
	 * @param string $crunch_name - Name of Crunch tag to be manipulated.
	 * @return boolean - Whether or not the specified Crunch tag exists.
	 */
	public static function tag_exists($crunch_id, $crunch_name)
	{
		global $db;

		$sql = 'SELECT COUNT(tag_id) AS tags 
			FROM ' . CRUNCH_TAGS_TABLE . '
				WHERE crunch_id = ' . (int) $crunch_id . "
				AND tag_word = '" . $db->sql_escape($crunch_name) . "'";
		$result = $db->sql_query($sql);

		$results = (int) $db->sql_fetchfield('tags');
		$db->sql_freeresult($result);

		return (($results > 0) ? true : false);
	}

	/**
	 * Checks to see if the Crunch tag word uses allowed characters.  Only A-Z, a-z, numbers, hyphens, and underscores allowed.
	 * @param string $crunch_name - Name of Crunch tag to be manipulated.
	 * @return boolean - Whether or not the Crunch tag uses allowed characters for the Crunch word.
	 */
	public static function check_tag_format($crunch_name)
	{
		return preg_match('/^[a-zA-Z0-9_\-]+$/', $crunch_name);  // _NO_ UTF-8 in the tag word!
	}

	/**
	 * Loads Crunch installation information.
	 * @return void
	 */
	public static function setup()
	{
		global $user;
		$crunch_install = 'Q3J1bmNoIDxzdHJvbmcgdGl0bGU9IiUxJHMiPnYlMiRzPC9zdHJvbmc+ICZj';
		$crunch_install .= 'b3B5OyAyMDA5IDxhIGhyZWY9Imh0dHA6Ly93d3cuaW5maW5pdHlob3VzZS5v';
		$crunch_install .= 'cmcvIiBzdHlsZT0iZm9udC13ZWlnaHQ6IGJvbGQ7Ij5PYnNpZGlhbjwvYT4=';
		define('CRUNCH_INSTALL', base64_decode($crunch_install));
		if(!defined('crunch::CHECKSUM'))
		{
			self::fail();
		}
		$user->lang['TRANSLATION_INFO'] = $user->lang['TRANSLATION_INFO'] . (($user->lang['TRANSLATION_INFO'] != '') ? '<br />' : '') . sprintf(CRUNCH_INSTALL, CRUNCH_BIG_VERSION, CRUNCH_VERSION);
	}

	/**
	 * Emergency function.
	 * @return void
	 */
	public static function fail()
	{
		global $template;
		$template->assign_var('DISABLE_CRUNCH', true);
		$template->assign_var('CRUNCH_FAIL', base64_decode(self::FAIL));
	}

	/**
	 * Checks to see if the installed version of Crunch is current.
	 * @param boolean &$up_to_date - Whether Crunch is up to date or not
	 * @param string &$latest_version - The latest version
	 * @param string &$announcement_url - The URL that the latest version is announced at
	 * @return void
	 */
	public static function version(&$up_to_date, &$latest_version, &$announcement_url)
	{
		global $user, $config;

		// Check the version, load our remote version check file!
		$errstr = '';
		$errno = 0;
		$info = get_remote_file('my-svn.assembla.com', '/svn/obsidian_mods/versions', ((!self::$dev) ? 'crunch.txt' : 'crunch_dev.txt'), $errstr, $errno);

		if ($info === false)
		{
			trigger_error($errstr, E_USER_WARNING);
		}

		$info = explode("\n", $info);
		$latest_version = trim($info[0]);
		$announcement_url = htmlspecialchars(trim($info[1]));
		$up_to_date = (!version_compare(str_replace('rc', 'RC', strtolower($config['crunch_version'])), str_replace('rc', 'RC', strtolower($latest_version)), '<'));
	}
}

?>