<?php
//Schema data file for create_schema_files.php
//Just to make using it a little easier...

/**
* Define the basic structure
* The format:
*		array('{TABLE_NAME}' => {TABLE_DATA})
*		{TABLE_DATA}:
*			COLUMNS = array({column_name} = array({column_type}, {default}, {auto_increment}))
*			PRIMARY_KEY = {column_name(s)}
*			KEYS = array({key_name} = array({key_type}, {column_name(s)})),
*
*	Column Types:
*	INT:x		=> SIGNED int(x)
*	BINT		=> BIGINT
*	UINT		=> mediumint(8) UNSIGNED
*	UINT:x		=> int(x) UNSIGNED
*	TINT:x		=> tinyint(x)
*	USINT		=> smallint(4) UNSIGNED (for _order columns)
*	BOOL		=> tinyint(1) UNSIGNED
*	VCHAR		=> varchar(255)
*	CHAR:x		=> char(x)
*	XSTEXT_UNI	=> text for storing 100 characters (topic_title for example)
*	STEXT_UNI	=> text for storing 255 characters (normal input field with a max of 255 single-byte chars) - same as VCHAR_UNI
*	TEXT_UNI	=> text for storing 3000 characters (short text, descriptions, comments, etc.)
*	MTEXT_UNI	=> mediumtext (post text, large text)
*	VCHAR:x		=> varchar(x)
*	TIMESTAMP	=> int(11) UNSIGNED
*	DECIMAL		=> decimal number (5,2)
*	DECIMAL:	=> decimal number (x,2)
*	PDECIMAL	=> precision decimal number (6,3)
*	PDECIMAL:	=> precision decimal number (x,3)
*	VCHAR_UNI	=> varchar(255) BINARY
*	VCHAR_CI	=> varchar_ci for postgresql, others VCHAR
*/
function get_schema_struct()
{
	$schema_data = array();

	$schema_data['phpbb_crunch'] = array(
		'COLUMNS'		=> array(
			'crunch_id'					=> array('UINT', NULL, 'auto_increment'),
			'poster_id'					=> array('UINT', 0),
//			'crunch_username'			=> array('VCHAR_UNI:255', ''),
			'crunch_subject'			=> array('STEXT_UNI', '', 'true_sort'),
			'crunch_text'				=> array('MTEXT_UNI', ''),
			'crunch_options'			=> array('UINT:11', 7),
			'crunch_bitfield'			=> array('VCHAR:255', ''),
			'crunch_uid'				=> array('VCHAR:8', ''),
		),
		'PRIMARY_KEY'	=> 'crunch_id',
	);
	
	$schema_data['phpbb_crunch_tags'] = array(
		'COLUMNS'		=> array(
			'tag_id'				=> array('UINT', NULL, 'auto_increment'),
			'crunch_id'				=> array('UINT', 0),
			'tag_word'				=> array('STEXT_UNI', ''),
			'tag_enable'			=> array('BOOL', 1),
		),
		'PRIMARY_KEY'	=> 'tag_id',
	);
	
	return $schema_data;
}

?>